@@ -89,7 +89,7 @@ using namespace std;
 #define WD 0x20        /* Action D Write access allowed */
 #define RD 0x40        /* Action D Read access allowed */
 #define R7 0x80        /* Action 7 Read access allowed */
-static const char _dat79Dv[]={
+static const unsigned char _dat79Dv[]={
 NDF_HEADER(0x20, 5),
 /* Number of variables: */ 0x25,
 
@@ -163,8 +163,8 @@ static const char _datB[]="\x01\x02"
 #define WORD 0x84                    /* Spritecount is larger than a byte and is defined using W() */
 #define OFFSET 0x88                  /* Allow replacing sprite ranges with offsets */
 #define W(cnt) cnt & 0xFF, cnt >> 8  /* Construct word count */
-static const char _dat5[]={
-NDF_HEADER(0x04, 11),
+static const unsigned char _dat5[]={
+NDF_HEADER(0x04, 12),
 /*04*/ OFFSET,          OPTIONS(3), 0x30, 0x70, 0xF0,
 /*05*/ OFFSET,          OPTIONS(1), 0x30,
 /*06*/ OFFSET,          OPTIONS(2), 0x4A, 0x5A,
@@ -184,6 +184,7 @@ NDF_HEADER(0x04, 11),
 /*14*/ OFFSET,          OPTIONS(1), 0x24,
 /*15*/ OFFSET,          OPTIONS(1), 0xA2,
 /*16*/ OFFSET,          OPTIONS(1), 0x09,
+/*17*/ OFFSET,          OPTIONS(1), 0x10,
 00,
 NDF_END
 };
@@ -236,9 +237,9 @@ static const char _datTextIDs[]="\x04\x09"
 #define NONE 0x80
 #define GROUNDVEHICLE MASK(TRAIN) | MASK(ROADVEH)
 #define VEHICLE MASK(TRAIN) | MASK(ROADVEH) | MASK(SHIP) | MASK(AIRCRAFT)
-static const char _datcallbacks[]={
-NDF_HEADER(0x05, 20),
-/* Count: */ W(0x15E),
+static const unsigned char _datcallbacks[]={
+NDF_HEADER(0x05, 21),
+/* Count: */ W(0x15F),
 /* 00*/ NONE, NONE, NONE, NONE, NONE, NONE, NONE, NONE, NONE, NONE, NONE, NONE, NONE, NONE, NONE, NONE,
 /* 10*/ GROUNDVEHICLE | MASK(SHIP), GROUNDVEHICLE, VEHICLE, STATION, STATION, VEHICLE, GROUNDVEHICLE, HOUSE,
 /* 18*/ MASK(STATION) | VEHICLE, VEHICLE, HOUSE, HOUSE, HOUSE, TRAIN, HOUSE, HOUSE,
@@ -265,7 +266,7 @@ NDF_HEADER(0x05, 20),
 /*140*/ STATION, STATION, STATION, HOUSE, SOUND, CARGO, SIGNAL, CANAL,
 /*148*/ HOUSE, STATION, INDUSTRY, INDUSTRY, INDUSTRY, HOUSE, HOUSE, HOUSE,
 /*150*/ AIRTILE, NONE, AIRTILE, AIRTILE, AIRTILE, AIRPORT, AIRPORT, OBJECT,
-/*158*/ OBJECT, OBJECT, OBJECT, OBJECT, OBJECT, OBJECT,
+/*158*/ OBJECT, OBJECT, OBJECT, OBJECT, OBJECT, OBJECT, VEHICLE,
 NDF_END
 };
 #undef W
@@ -292,52 +293,52 @@ NDF_END
 #undef GROUNDVEHICLE
 #undef VEHICLE
 
-/*	Languages
-	=========
-
-	Names of languages by ID (empty if not defined), each terminated by newline
-*/
-static const char _datlangs[]="\x00\x05"
-//		x0/x4/x8/xC		x1/x5/x9/xD		x2/x6/xA/xE		x3/x7/xB/xF
-/*0x*/	"American\n"	"English\n"		"German\n"		"French\n"
-/*0x*/	"Spanish\n"		"Esperanto\n"	"Ido\n"			"Russian\n"
-/*0x*/	"Irish\n"		"Maltese\n"		"Tamil\n"		"Chuvash\n"
-/*0x*/	"Chinese (Trad)\n" "Serbian\n" "Norwegian (Nynorsk)\n" "Welsh\n"
-//		x0/x4/x8/xC		x1/x5/x9/xD		x2/x6/xA/xE		x3/x7/xB/xF
-/*1x*/	"Belarusian\n"	"Marathi\n"		"Faroese\n"		"\n"
-/*1x*/	"Arabic (Egypt)\n" "Czech\n"	"Slovak\n"		"\n"
-/*1x*/	"Bulgarian\n"	"\n"			"\n"			"Afrikaans\n"
-/*1x*/	"\n"			"\n"			"Greek\n"		"Dutch\n"
-//		x0/x4/x8/xC		x1/x5/x9/xD		x2/x6/xA/xE		x3/x7/xB/xF
-/*2x*/	"\n"			"Basque\n"		"Catalan\n"		"Luxembourgish\n"
-/*2x*/	"Hungarian\n"	"\n"			"Macedonian\n"	"Italian\n"
-/*2x*/	"Romanian\n"	"Icelandic\n"	"Latvian\n"		"Lithuanian\n"
-/*2x*/	"Slovenian\n"	"Danish\n"		"Swedish\n"		"Norwegian (Bokmal)\n"
-//		x0/x4/x8/xC		x1/x5/x9/xD		x2/x6/xA/xE		x3/x7/xB/xF
-/*3x*/	"Polish\n"		"Galician\n"	"Frisian\n"		"Ukrainian\n"
-/*3x*/	"Estonian\n"	"Finnish\n"		"Portuguese\n"	"Brazilian Portuguese\n"
-/*3x*/	"Croatian\n"	"Japanese\n"	"Korean\n"		"\n"
-/*3x*/	"Malay\n"		"\n"			"Turkish\n"		"\n"
-//		x0/x4/x8/xC		x1/x5/x9/xD		x2/x6/xA/xE		x3/x7/xB/xF
-/*4x*/	"\n"			"\n"			"Thai\n"		"\n"
-/*4x*/	"\n"			"\n"			"\n"			"\n"
-/*4x*/	"\n"			"\n"			"\n"			"\n"
-/*4x*/	"\n"			"\n"			"\n"			"\n"
-//		x0/x4/x8/xC		x1/x5/x9/xD		x2/x6/xA/xE		x3/x7/xB/xF
-/*5x*/	"\n"			"\n"			"\n"			"\n"
-/*5x*/	"Vietnamese\n"	"\n"			"Chinese (Simp)\n" "\n"
-/*5x*/	"\n"			"\n"			"Indonesian\n"	"\n"
-/*5x*/	"Urdu\n"		"\n"			"\n"			"\n"
-//		x0/x4/x8/xC		x1/x5/x9/xD		x2/x6/xA/xE		x3/x7/xB/xF
-/*6x*/	"\n"			"Hebrew\n"		"Persian\n"		"\n"
-/*6x*/	"\n"			"\n"			"\n"			"\n"
-/*6x*/	"\n"			"\n"			"\n"			"\n"
-/*6x*/	"\n"			"\n"			"\n"			"\n"
-//		x0/x4/x8/xC		x1/x5/x9/xD		x2/x6/xA/xE		x3/x7/xB/xF
-/*7x*/	"\n"			"\n"			"\n"			"\n"
-/*7x*/	"\n"			"\n"			"\n"			"\n"
-/*7x*/	"\n"			"\n"			"\n"			"\n"
-/*7x*/	"\n"			"\n"			"\n"			"any\n"
+/* Languages
+ * ========
+ *
+ * Names of languages by ID (empty if not defined), each terminated by newline
+ */
+static const char _datlangs[]="\x00\x06"
+//      x0/x4/x8/xC               x1/x5/x9/xD               x2/x6/xA/xE               x3/x7/xB/xF
+/*0x*/  "American\n"              "English\n"               "German\n"                "French\n"
+/*0x*/  "Spanish\n"               "Esperanto\n"             "Ido\n"                   "Russian\n"
+/*0x*/  "Irish\n"                 "Maltese\n"               "Tamil\n"                 "Chuvash\n"
+/*0x*/  "Chinese (Traditional)\n" "Serbian\n"               "Norwegian (Nynorsk)\n"   "Welsh\n"
+//      x0/x4/x8/xC               x1/x5/x9/xD               x2/x6/xA/xE               x3/x7/xB/xF
+/*1x*/  "Belarusian\n"            "Marathi\n"               "Faroese\n"               "\n"
+/*1x*/  "Arabic (Egypt)\n"        "Czech\n"                 "Slovak\n"                "\n"
+/*1x*/  "Bulgarian\n"             "\n"                      "\n"                      "Afrikaans\n"
+/*1x*/  "\n"                      "\n"                      "Greek\n"                 "Dutch\n"
+//      x0/x4/x8/xC               x1/x5/x9/xD               x2/x6/xA/xE               x3/x7/xB/xF
+/*2x*/  "\n"                      "Basque\n"                "Catalan\n"               "Luxembourgish\n"
+/*2x*/  "Hungarian\n"             "\n"                      "Macedonian\n"            "Italian\n"
+/*2x*/  "Romanian\n"              "Icelandic\n"             "Latvian\n"               "Lithuanian\n"
+/*2x*/  "Slovenian\n"             "Danish\n"                "Swedish\n"               "Norwegian (Bokmal)\n"
+//      x0/x4/x8/xC               x1/x5/x9/xD               x2/x6/xA/xE               x3/x7/xB/xF
+/*3x*/  "Polish\n"                "Galician\n"              "Frisian\n"               "Ukrainian\n"
+/*3x*/  "Estonian\n"              "Finnish\n"               "Portuguese\n"            "Brazilian Portuguese\n"
+/*3x*/  "Croatian\n"              "Japanese\n"              "Korean\n"                "\n"
+/*3x*/  "Malay\n"                 "Australian\n"            "Turkish\n"               "\n"
+//      x0/x4/x8/xC               x1/x5/x9/xD               x2/x6/xA/xE               x3/x7/xB/xF
+/*4x*/  "\n"                      "\n"                      "Thai\n"                  "\n"
+/*4x*/  "\n"                      "\n"                      "\n"                      "\n"
+/*4x*/  "\n"                      "\n"                      "\n"                      "\n"
+/*4x*/  "\n"                      "\n"                      "\n"                      "\n"
+//      x0/x4/x8/xC               x1/x5/x9/xD               x2/x6/xA/xE               x3/x7/xB/xF
+/*5x*/  "\n"                      "\n"                      "\n"                      "\n"
+/*5x*/  "Vietnamese\n"            "\n"                      "Chinese (Simplified)\n"  "\n"
+/*5x*/  "\n"                      "\n"                      "Indonesian\n"            "\n"
+/*5x*/  "Urdu\n"                  "\n"                      "\n"                      "\n"
+//      x0/x4/x8/xC               x1/x5/x9/xD               x2/x6/xA/xE               x3/x7/xB/xF
+/*6x*/  "\n"                      "Hebrew\n"                "Persian\n"               "\n"
+/*6x*/  "\n"                      "\n"                      "\n"                      "\n"
+/*6x*/  "\n"                      "\n"                      "\n"                      "\n"
+/*6x*/  "\n"                      "\n"                      "\n"                      "\n"
+//      x0/x4/x8/xC               x1/x5/x9/xD               x2/x6/xA/xE               x3/x7/xB/xF
+/*7x*/  "\n"                      "\n"                      "\n"                      "\n"
+/*7x*/  "\n"                      "\n"                      "\n"                      "\n"
+/*7x*/  "\n"                      "\n"                      "\n"                      "\n"
+/*7x*/  "\n"                      "\n"                      "\n"                      "any\n"
 ;
 
 /*	Version check data
@@ -367,7 +368,7 @@ static const char _datversions[]="\x00\x04"
 #define ACT2_HOUSE 0x03
 #define ACT2_INDPROD 0x04
 #define ACT2_NONE 0xFF
-static const char _datfeat[]={
+static const unsigned char _datfeat[]={
 NDF_HEADER(0x12, 3),
 /* Max. feature: */ 0x11,
 
@@ -468,7 +469,7 @@ NDF_END
  * May be specified alone, or combined with any of the BYTE/EXTBYTE/WORD/DWORD datas */
 #define LINEBREAK 0xC0
 
-/* Is replaced by the data following after the next END. Mulitple SUBDATA are resolved in a LIFO style. */
+/* Is replaced by the data following after the next END. Mulitple SUBDATA are resolved in a depth-first style. */
 #define SUBDATA 0xFE
 
 /* Raw byte to appear in the nfo */
@@ -484,8 +485,8 @@ NDF_END
 /* Repeat 'data' until a certain terminator with length 'length' is encountered */
 #define REPEAT_UNTIL(data, length, ...) '*', data, length, __VA_ARGS__
 
-static const char _dat0[]={
-NDF_HEADER(0x0D, 15),
+static const unsigned char _dat0[]={
+NDF_HEADER(0x0D, 21),
 /*Maximum feature:*/ 0x11,
 // Feature 00:
 /*00*/ WORD | DATE, INVALID, BYTE, BYTE | DECIMAL, BYTE | DECIMAL, BYTE, BYTE, BYTE | DECIMAL,
@@ -493,30 +494,46 @@ NDF_HEADER(0x0D, 15),
 /*10*/ INVALID, INVALID, BYTE, BYTE, BYTE | DECIMAL, BYTE, BYTE, BYTE,
 /*18*/ BYTE, BYTE, EXTBYTE, WORD | DECIMAL, BYTE, DWORD | HEX, BYTE, BYTE,
 /*20*/ BYTE, BYTE, BYTE, BYTE | DECIMAL, BYTE, BYTE, BYTE | DECIMAL, BYTE,
-/*28*/ WORD | HEX, WORD | HEX, DWORD | DATE,
+/*28*/ WORD | HEX, WORD | HEX, DWORD | DATE, WORD, SUBDATA, SUBDATA,
 END,
+// Subdata - prop 2C:
+BYTE, REPEAT_N(BYTE, DATAFROM(0)), END,
+// Subdata - prop 2D:
+BYTE, REPEAT_N(BYTE, DATAFROM(0)), END,
 
 // Feature 01:
 /*00*/ WORD | DATE, INVALID, BYTE, BYTE | DECIMAL, BYTE | DECIMAL, INVALID, BYTE, BYTE | DECIMAL,
 /*08*/ BYTE | DECIMAL, BYTE, DWORD, INVALID, INVALID, INVALID, BYTE, BYTE | DECIMAL,
 /*10*/ BYTE, BYTE, BYTE, BYTE | DECIMAL, BYTE | DECIMAL, BYTE | DECIMAL, DWORD | HEX, BYTE,
 /*18*/ BYTE, BYTE, BYTE, BYTE | DECIMAL, BYTE, WORD | HEX, WORD | HEX, DWORD | DATE,
-/*20*/ EXTBYTE, BYTE,
+/*20*/ EXTBYTE, BYTE, WORD, BYTE, SUBDATA, SUBDATA,
 END,
+// Subdata - prop 24:
+BYTE, REPEAT_N(BYTE, DATAFROM(0)), END,
+// Subdata - prop 25:
+BYTE, REPEAT_N(BYTE, DATAFROM(0)), END,
 
 // Feature 02:
 /*00*/ WORD | DATE, INVALID, BYTE, BYTE | DECIMAL, BYTE | DECIMAL, INVALID, BYTE, BYTE | DECIMAL,
 /*08*/ BYTE, BYTE, BYTE, BYTE | DECIMAL, BYTE, WORD | DECIMAL, INVALID, BYTE,
 /*10*/ BYTE, DWORD | HEX, BYTE, BYTE, BYTE, BYTE, BYTE | DECIMAL, BYTE,
-/*18*/ WORD | HEX, WORD | HEX, DWORD | DATE, EXTBYTE, BYTE,
+/*18*/ WORD | HEX, WORD | HEX, DWORD | DATE, EXTBYTE, BYTE, WORD, SUBDATA, SUBDATA,
 END,
+// Subdata - prop 1E:
+BYTE, REPEAT_N(BYTE, DATAFROM(0)), END,
+// Subdata - prop 1F:
+BYTE, REPEAT_N(BYTE, DATAFROM(0)), END,
 
 // Feature 03:
 /*00*/ WORD | DATE, INVALID, BYTE, BYTE | DECIMAL, BYTE | DECIMAL, INVALID, BYTE, BYTE | DECIMAL,
 /*08*/ BYTE, BYTE, BYTE, BYTE, BYTE | DECIMAL, BYTE, BYTE, WORD | DECIMAL,
 /*10*/ INVALID, BYTE | DECIMAL, BYTE, DWORD | HEX, BYTE, BYTE, BYTE | DECIMAL, BYTE,
-/*18*/ WORD | HEX, WORD | HEX, DWORD | DATE, EXTBYTE,
+/*18*/ WORD | HEX, WORD | HEX, DWORD | DATE, EXTBYTE, WORD, SUBDATA, SUBDATA, WORD,
 END,
+// Subdata - prop 1D:
+BYTE, REPEAT_N(BYTE, DATAFROM(0)), END,
+// Subdata - prop 1E:
+BYTE, REPEAT_N(BYTE, DATAFROM(0)), END,
 
 // Feature 04:
 /*00*/ INVALID, INVALID, INVALID, INVALID, INVALID, INVALID, INVALID, INVALID,
@@ -609,7 +626,7 @@ REPEAT_N(BYTE, 3), END,
 /*00*/ INVALID, INVALID, INVALID, INVALID, INVALID, INVALID, INVALID, INVALID,
 /*08*/ BYTE | DECIMAL, TEXTID, TEXTID, TEXTID, TEXTID, TEXTID, WORD | HEX, BYTE,
 /*10*/ BYTE, BYTE, DWORD | HEX, BYTE, BYTE, BYTE, WORD | HEX, DWORD | QUOTED,
-/*18*/ BYTE, WORD | HEX, BYTE, TEXTID, TEXTID,
+/*18*/ BYTE, WORD | HEX, BYTE, TEXTID, TEXTID, WORD,
 END,
 
 // Feature 0C:
@@ -620,7 +637,7 @@ END,
 // Feature 0D:
 /*00*/ INVALID, INVALID, INVALID, INVALID, INVALID, INVALID, INVALID, INVALID,
 /*08*/ BYTE, DWORD, SUBDATA, INVALID, SUBDATA, BYTE, BYTE, BYTE,
-/*10*/ WORD,
+/*10*/ WORD, WORD,
 END,
 // Subdata - prop 0A:
 BYTE, REMAININGSIZE | LINEBREAK, REPEAT_N(SUBDATA, DATAFROM(0)), END,
@@ -645,7 +662,7 @@ END,
 /*00*/ INVALID, INVALID, INVALID, INVALID, INVALID, INVALID, INVALID, INVALID,
 /*08*/ DWORD | QUOTED, WORD, WORD, WORD, WORD, WORD, SUBDATA, SUBDATA,
 /*10*/ BYTE, BYTE, BYTE, WORD, WORD, BYTE, BYTE, DWORD | DATE,
-/*18*/ SUBDATA, SUBDATA, BYTE,
+/*18*/ SUBDATA, SUBDATA, BYTE, WORD, WORD, SUBDATA,
 END,
 // Subdata - prop 0E:
 BYTE, REPEAT_N(DWORD | QUOTED, DATAFROM(0)), END,
@@ -655,6 +672,8 @@ BYTE, REPEAT_N(DWORD | QUOTED, DATAFROM(0)), END,
 BYTE, REPEAT_N(DWORD | QUOTED, DATAFROM(0)), END,
 // Subdata - prop 19:
 BYTE, REPEAT_N(DWORD | QUOTED, DATAFROM(0)), END,
+// Subdata - prop 1D:
+BYTE, REPEAT_N(DWORD | QUOTED, DATAFROM(0)), END,
 
 // Feature 11:
 /*00*/ INVALID, INVALID, INVALID, INVALID, INVALID, INVALID, INVALID, INVALID,
@@ -700,8 +719,8 @@ NDF_END
 #define DWORD6x(var, max) VAR6x(var, 4, max)
 #define LASTVAR80(var) (var == 0xFF ? 0xFF : var + 1), (var == 0xFF ? 0xF0 : 0x80)
 #define NOVAR80 LASTVAR80(0x7F)
-static const char _dat2v[]={
-NDF_HEADER(0x0D, 25),
+static const unsigned char _dat2v[]={
+NDF_HEADER(0x0D, 26),
 /*Maximum feature:*/ 0x11,
 /*Maximum operator:*/ 0x16,
 // Global variables:
@@ -715,7 +734,7 @@ LASTVAR80(0xFF),
 /*Related:*/ 0x00,
 DWORD(0x25),
 VAR(0x40, 3), VAR(0x41, 3), DWORD(0x42), DWORD(0x43), VAR(0x45, 3), DWORD(0x46), DWORD(0x47),
-BYTE(0x48), DWORD(0x49), DWORD(0x4A),
+BYTE(0x48), DWORD(0x49), DWORD(0x4A), DWORD(0x4B),
 WORD(0x5F),
 BYTE6x(0x60, 0x73),
 LASTVAR80(0xFF),
@@ -800,7 +819,7 @@ DWORD6x(0x64, 0xFF),
 NOVAR80,
 // Feature 10:
 /*Related:*/ 0x10,
-BYTE(0x40), BYTE(0x41), BYTE(0x42), DWORD(0x43),
+BYTE(0x40), BYTE(0x41), BYTE(0x42), DWORD(0x43), BYTE(0x44),
 WORD(0x5F),
 NOVAR80,
 // Feature 11:
@@ -824,10 +843,10 @@ NDF_END
 #undef NOVAR80
 
 #define W(cnt) cnt & 0xFF, cnt >> 8  /* Construct word count */
-static const char _datD[]={
-NDF_HEADER(0x14, 5),
+static const unsigned char _datD[]={
+NDF_HEADER(0x14, 6),
 /*Maximum feature:*/ 0x11,
-/*Max patch variable:*/ 0x13,
+/*Max patch variable:*/ 0x14,
 /*Max operator:*/ 0x0C,
 // GRM count:
 /*00*/W(0x74),
@@ -851,7 +870,7 @@ NDF_HEADER(0x14, 5),
 NDF_END
 };
 
-static const char _datIDs[]={
+static const unsigned char _datIDs[]={
 NDF_HEADER(0x77, 4),
 /*Maximum feature:*/ 0x11,
 /*00*/W(0x73),
@@ -877,7 +896,7 @@ NDF_END
 #undef W
 
 /*	Action 4 strings */
-static const char _dat4[]={
+static const unsigned char _dat4[]={
 NDF_HEADER(0x03, 5),
 /*Maximum feature:*/ 0x11,
 // Rules for one-byte IDs:
@@ -924,7 +943,7 @@ NDF_END
 
 /*	Randomized action 2 */
 #define ACT2RANDOM(self_bits, related_bits, num_triggers) self_bits, related_bits, num_triggers
-static const char _dat2r[]={
+static const unsigned char _dat2r[]={
 NDF_HEADER(0x02, 6),
 /*Maximum feature:*/ 0x11,
 /*00*/ACT2RANDOM( 8,  8, 5),
