@@ -81,7 +81,7 @@ static void usage(void)
 		"    -x        Disable production of unquoted escape sequences.\n"
 		"    -xx       Disable production of both quoted and unquoted escape sequences.\n"
 		"              This has the side effect of producing a version 6 .nfo, instead\n"
-		"              of a version 7 .nfo.\n"
+		"              of a version 32 .nfo.\n"
 		"    -X        List sprite numbers in the PCX file in hex.\n"
 		"\n"
 		"Options for encoding:\n"
@@ -89,6 +89,8 @@ static void usage(void)
 		"    -u        Save uncompressed data (probably not a good idea)\n"
 		"    -q        Suppress warning messages\n"
 		"    -s        Suppress progress output\n"
+		"    -g <num>  Version of the encoded container format (default 1, maximum 2)\n"
+		"    -n        Try both compression algorithms and choose the most efficient\n"
 		"\n"
 		"Options for both encoding and decoding:\n"
 		"    -m <num>  Apply colour translation to all sprites except character-glyphs.\n"
@@ -199,6 +201,7 @@ static const defpal defpals[] =
 static int *colourmaps[] = { palmap0, palmap1 };
 
 bool _interactive;
+bool _best_compression = false;
 
 static int movetoreal(char *newfile, char *realfile)
 {
@@ -251,12 +254,12 @@ SpriteSheetFormat _outputformat = SSF_PNG;
 SpriteSheetFormat _outputformat = SSF_PCX;
 #endif
 
-const char * getoutputext()
+const char * getoutputext(bool rgba)
 {
 	switch (_outputformat) {
 #ifdef WITH_PNG
 		case SSF_PNG:
-			return ".png";
+			return rgba ? "32.png" : ".png";
 #endif
 		case SSF_PCX:
 		default:
@@ -267,7 +270,7 @@ const char * getoutputext()
 class spritefiles : public multifile {
 public:
 	spritefiles() { init(); }
-	spritefiles(const char *basename, const char *directory);
+	spritefiles(const char *basename, const char *directory, bool rgba);
 	virtual FILE *curfile()  { return thecurfile; }
 	virtual FILE *nextfile();
 	virtual const char *filename() { return thecurfilename; }
@@ -278,19 +281,22 @@ private:
 		basename = thecurfilename = NULL;
 		directory = NULL;
 		filenum = 0;
+		rgba=false;
 	}
 		FILE *thecurfile;
 		char *thecurfilename;
 		const char *directory;
 		const char *basename;
 		unsigned int filenum;
+		bool rgba;
 };
 
-spritefiles::spritefiles(const char *basename, const char *directory)
+spritefiles::spritefiles(const char *basename, const char *directory, bool rgba)
 {
 	init();
 	spritefiles::basename = basename;
 	spritefiles::directory = directory;
+	spritefiles::rgba=rgba;
 }
 
 FILE *spritefiles::nextfile()
@@ -298,7 +304,7 @@ FILE *spritefiles::nextfile()
 	FILE *oldfile = thecurfile;
 	char *oldname = thecurfilename;
 
-	thecurfilename = strdup(spritefilename(basename, directory, getoutputext(), filenum++, "wb", 1));
+	thecurfilename = strdup(spritefilename(basename, directory, getoutputext(rgba), filenum++, "wb", 1));
 	thecurfile = fopen(thecurfilename, "wb");
 
 	if (thecurfile) {	// new open succeeded, close old one
@@ -317,7 +323,13 @@ FILE *spritefiles::nextfile()
 int _crop=0;
 int _quiet=0;
 
-static int encode(const char *file, const char *dir, int compress, int *colourmap)
+static const char header[] = {
+	'\x00', '\x00',                 // End-of-file marker for old OTTDp versions
+	'G',    'R',    'F',    '\x82', // Container version 2
+	'\x0D', '\x0A', '\x1A', '\x0A', // Detect garbled transmission
+};
+
+static int encode(const char *file, const char *dir, int compress, int *colourmap, int grfcontversion)
 {
 	char *grfnew, *infofile;
 	FILE *grf;
@@ -337,219 +349,297 @@ static int encode(const char *file, const char *dir, int compress, int *colourma
 	long totaluncomp = 1, totalcomp = 1;
 	long totaltransp = 1, totaluntransp = 1;
 	long totalreg = 1, totalunreg = 1;
-	int spriteno = 0;
 
+	if (grfcontversion == 2) {
+		cfwrite("writing header", header, sizeof(header), 1, grf);
+		int size = 1 + 4; // The size of the zoom + end of data
+		for(int i = 0; i < info.size(); i++) {
+			switch(info[i].GetType()){
+			case Sprite::ST_INCLUDE:
+			case Sprite::ST_REAL:
+				size += 4 + 1 + 4; // Size + type + ID
+				break;
 
-	for(int i=0;i<info.size();i++){
-		//int comp1 = totalcomp / totaluncomp;
-		int comp2 = (100L * totalcomp / totaluncomp);// % 100;
-
-		//int comp3 = totaltransp / totaluntransp;
-		int comp4 = (100L * totaltransp / totaluntransp);// % 100;
-
-		//int comp5 = totalreg / totalunreg;
-		int comp6 = (100L * totalreg / totalunreg);// % 100;
-
-		if (_interactive) {
-			printf("\rSprite%5d  Done:%3d%%  "
-				"Compressed:%3d%% (Transparency:%3d%%, Redundancy:%3d%%)\r",
-				spriteno, (int) (i*100L/info.size()),
-				comp2, comp4, comp6);
-		}
-
-		switch(info[i].GetType()){
-		case Sprite::ST_INCLUDE:{
-			static const char *action = "copying binary blob";
-			const char *bininclude=((const Include&)info[i]).GetName();
-			FILE *bin = fopen(bininclude, "rb");
-			if (!bin) {
-				fperror("%s:%i: Error: Cannot include %s: Could not open.\n", file, i, bininclude);
-				exit(2);
-			}
-
-			struct stat stat_buf;
-			if ( fstat(fileno(bin), &stat_buf) ) {
-				fperror("%s:%i: Error: Could not stat %s.\n", file, i, bininclude);
-				exit(2);
-			}
-			if ( stat_buf.st_mode & S_IFDIR ) {
-				fprintf(stderr, "%s:%i: Error: Cannot include %s: Is a directory.\n", file, i, bininclude);
-				exit(2);
-			}
-			off_t fsize = stat_buf.st_size;
-
-			const char *nameofs = bininclude + strlen(bininclude);
-			while (nameofs > bininclude) {
-				nameofs--;
-				if (nameofs[0] == '\\' || nameofs[0] == '/') {
-					nameofs++;
-					break;
-				}
-			}
-			int namelen = strlen(nameofs);
-			if (namelen > 255) {
-				fprintf(stderr, "%s:%i: Error: binary include has too long filename %s\n", file, i, nameofs);
-				exit(2);
-			}
-
-			int spritesize = 3 + namelen + fsize;
-			if (spritesize < 5) {
-				fprintf(stderr, "%s:%i: Error: binary include %s is empty\n", file, i, nameofs);
-				exit(2);
-			}
-			if (spritesize > 65535) {
-				fprintf(stderr, "%s:%i: Error: binary include %s is too large\n", file, i, nameofs);
-				exit(2);
-			}
-
-			totalcomp += spritesize;
-			totaluncomp += spritesize;
-			spriteno++;
-
-			U16 le_spritesize = BE_SWAP16(spritesize);
-			cfwrite(action, &le_spritesize, 1, 2, grf);
-			fputc(0xff, grf);
-			fputc(0xff, grf);
-			fputc(namelen, grf);
-			cfwrite(action, nameofs, namelen+1, 1, grf);
-
-			char *buffer = new char[16384];
-			while (fsize > 0) {
-				int chunk = 16384;
-				if (chunk > fsize) chunk=fsize;
-				cfread(action, buffer, chunk, 1, bin);
-				cfwrite(action, buffer, chunk, 1, grf);
-				fsize -= chunk;
-			}
-			delete[]buffer;
-			fclose(bin);
-		}
-			break;
-		case Sprite::ST_PSEUDO:{
-			static const char *action = "writing pseudo sprite";
-			const Pseudo&sprite=(const Pseudo&)info[i];
-			U16 size=sprite.size();
-			totalcomp += size;
-			totaluncomp += size;
-			spriteno++;
-
-			U16 le_size = BE_SWAP16(size);
-			cfwrite(action, &le_size, 1, 2, grf);
-			fputc(0xff, grf);
-			cfwrite(action, sprite.GetData(),1,size,grf);
-			if(spriteno == 1 && sprite.size() == 4){
-				int reported = *((S32*)sprite.GetData());
-				reported = BE_SWAP32(reported) + 1;
-				if(reported != info.size() && !_quiet)
-					fprintf(stderr, "%s:1: Warning: Found %d %s sprites than sprite 0 reports.\n",
-					file,
-					abs(info.size() - reported),
-					info.size()>reported?"more":"fewer");
+			case Sprite::ST_PSEUDO:
+				size += 4 + 1 + ((const Pseudo&)info[i]).size();
+				break;
 			}
 		}
-			break;
-		case Sprite::ST_REAL:{	// real sprite, encode it
-			const Real&sprite=(Real&)info[i];
-			info.PrepareReal(sprite);
-			U8 *image = (U8*) malloc(info.imgsize);
-			if (!image) {
-				fprintf(stderr, "%s:%d: Error: can't allocate sprite memory (%ld bytes)\n", file, i, info.imgsize);
-				exit(2);
-			}
-
-			info.getsprite(image);
-
-			int k=0;
-			for (int j=info.imgsize-1; j >= 0; j--)
-				if (image[j] == 0xFF) k++;
-
-			if (k && !_quiet)
-				fprintf(stderr, "%s:%d: Warning: %d of %ld pixels (%ld%%) are pure white\n",
-					file, i, k, info.imgsize, k*100/info.imgsize);
-
-			if(_crop && !DONOTCROP(info.inf.info)){
-				int i=0,j=0;
-				for(i=info.imgsize-1;i>=0;i--)if(image[i])break; // Find last non-blue pixel
-				if(i<0)// We've got an all-blue sprite
-					info.sx=info.sy=info.imgsize=1;
-				else{
-					i=info.imgsize-(i+info.sx-i%info.sx/*begining of next line*/);
-					info.sy-=i/info.sx;
-
-					for(i=0;i<info.imgsize;i++){
-						if(image[i])
-							break; // Find first non-blue pixel
-					}
-					i-=i%info.sx;// Move to beginning of line
-
-					info.sy-=i/info.sx;
-					info.inf.yrel+=i/info.sx;
-					if(i)memmove(image,image+i,info.imgsize-i);
-					for(i=0;i<info.sx;i++){
-						for(j=0;j<info.sy;j++){
-							if(image[i+j*info.sx])goto foundfirst;
-						}
-					}
-foundfirst:
-					if(i){
-						for(j=0;j<info.sy;j++)
-							memmove(image+j*(info.sx-i),image+j*info.sx+i,info.sx-i);
-						info.inf.xrel+=i;
-						info.sx-=i;
-					}
-
-					for(i=info.sx-1;i>=0;i--){
-						for(j=0;j<info.sy;j++){
-							if(image[i+j*info.sx])goto foundlast;
-						}
-					}
-foundlast:
-					i=info.sx-i-1;
-					if(i){
-						for(j=1;j<info.sy;j++)
-							memmove(image+j*(info.sx-i),image+j*info.sx,info.sx-i);
-						info.sx-=i;
-					}
-
-				}
-				info.inf.xdim = info.sx;
-				info.inf.ydim = info.sy;
-				info.imgsize = info.sx * info.sy;
-			}
-
-			U16 compsize;
-			if (HASTRANSPARENCY(info.inf.info)) {
-				compsize = encodetile(grf, image, info.imgsize, 0, info.sx, info.sy, info.inf, compress, spriteno);
-				totaltransp += getlasttilesize();	// how much after transparency removed
-				totaluntransp += info.imgsize;		// how much with transparency
-
-				totalreg += compsize;			// how much after transp&redund removed
-				totalunreg += getlasttilesize();	// how much with redund
-			} else {
-				compsize = encoderegular(grf, image, info.imgsize, info.inf, compress);
-				totaltransp += info.imgsize;
-				totaluntransp += info.imgsize;
-
-				totalreg += compsize;
-				totalunreg += info.imgsize;
-			}
-
-			totalcomp += compsize;
-			totaluncomp += info.imgsize;
-			spriteno++;
-			free(image);
-		}
-			break;
-		default:
-			fprintf(stderr, "%s:%d: Error: What type of sprite is that?", file, i);
-			exit(2);
-		}
+		writedword("writing header", size, grf);
+		fputc(0x00, grf); // Compression
 	}
 
-	U16 endoffile = 0;
-	U32 checksum = 0;
-	cfwrite("writing end-of-file", &endoffile, 1, 2, grf);
-	cfwrite("writing checksum", &checksum, 1, 4, grf);
+	for (int pass = 1; pass <= grfcontversion; pass++) {
+		for(int i=0;i<info.size();i++){
+			if (_interactive && pass != grfcontversion) {
+				int comp2 = (100L * totalcomp / totaluncomp);// % 100;
+				int comp4 = (100L * totaltransp / totaluntransp);// % 100;
+				int comp6 = (100L * totalreg / totalunreg);// % 100;
+				printf("\rSprite%5d  Done:%3d%%  "
+					"Compressed:%3d%% (Transparency:%3d%%, Redundancy:%3d%%)\r",
+					i, (int) (i*100L/info.size()),
+					comp2, comp4, comp6);
+			}
+
+			switch(info[i].GetType()){
+			case Sprite::ST_INCLUDE:{
+				static const char *action = "copying binary blob";
+				if (pass != grfcontversion) {
+					writespritesize(action, 4, grfcontversion, grf);
+					fputc(0xfd, grf);
+					writedword(action, i + 1, grf);
+					break;
+				}
+
+				const char *bininclude=((const Include&)info[i]).GetName();
+				FILE *bin = fopen(bininclude, "rb");
+				if (!bin) {
+					fperror("%s:%i: Error: Cannot include %s: Could not open.\n", file, i, bininclude);
+					exit(2);
+				}
+
+				struct stat stat_buf;
+				if ( fstat(fileno(bin), &stat_buf) ) {
+					fperror("%s:%i: Error: Could not stat %s.\n", file, i, bininclude);
+					exit(2);
+				}
+				if ( stat_buf.st_mode & S_IFDIR ) {
+					fprintf(stderr, "%s:%i: Error: Cannot include %s: Is a directory.\n", file, i, bininclude);
+					exit(2);
+				}
+				off_t fsize = stat_buf.st_size;
+
+				const char *nameofs = bininclude + strlen(bininclude);
+				while (nameofs > bininclude) {
+					nameofs--;
+					if (nameofs[0] == '\\' || nameofs[0] == '/') {
+						nameofs++;
+						break;
+					}
+				}
+				int namelen = strlen(nameofs);
+				if (namelen > 255) {
+					fprintf(stderr, "%s:%i: Error: binary include has too long filename %s\n", file, i, nameofs);
+					exit(2);
+				}
+
+				int spritesize = 3 + namelen + fsize;
+				if (spritesize < 5) {
+					fprintf(stderr, "%s:%i: Error: binary include %s is empty\n", file, i, nameofs);
+					exit(2);
+				}
+				if (grfcontversion > 1) spritesize++; // We need the first 0xFF to be accounted for as well.
+				/* GRF container version only allowed 64K, the rest prevents underflows. */
+				if ((grfcontversion == 1 && spritesize > 65535) || spritesize < fsize || spritesize < 0) {
+					fprintf(stderr, "%s:%i: Error: binary include %s is too large\n", file, i, nameofs);
+					exit(2);
+				}
+
+				totalcomp += spritesize;
+				totaluncomp += spritesize;
+
+				if (grfcontversion == 2) writedword(action, i + 1, grf);
+				writespritesize(action, spritesize, grfcontversion, grf);
+				fputc(0xff, grf);
+				fputc(0xff, grf);
+				fputc(namelen, grf);
+				cfwrite(action, nameofs, namelen+1, 1, grf);
+
+				char *buffer = new char[16384];
+				while (fsize > 0) {
+					int chunk = 16384;
+					if (chunk > fsize) chunk=fsize;
+					cfread(action, buffer, chunk, 1, bin);
+					cfwrite(action, buffer, chunk, 1, grf);
+					fsize -= chunk;
+				}
+				delete[]buffer;
+				fclose(bin);
+			}
+				break;
+			case Sprite::ST_PSEUDO:{
+				if (pass != 1) {
+					break;
+				}
+
+				static const char *action = "writing pseudo sprite";
+				const Pseudo&sprite=(const Pseudo&)info[i];
+				U16 size=sprite.size();
+				totalcomp += size;
+				totaluncomp += size;
+
+				writespritesize(action, size, grfcontversion, grf);
+				fputc(0xff, grf);
+				cfwrite(action, sprite.GetData(),1,size,grf);
+				if(i == 0 && sprite.size() == 4){
+					int reported = *((S32*)sprite.GetData());
+					reported = BE_SWAP32(reported) + 1;
+					if(reported != info.size() && !_quiet)
+						fprintf(stderr, "%s:1: Warning: Found %d %s sprites than sprite 0 reports.\n",
+						file,
+						abs(info.size() - reported),
+						info.size()>reported?"more":"fewer");
+				}
+			}
+				break;
+			case Sprite::ST_REAL:{	// real sprite, encode it
+				static const char *action = "writing real sprite";
+
+				const Real&sprite=(Real&)info[i];
+				for (size_t j = 0; j < (grfcontversion == 1 ? 1 : sprite.infs.size()); j++) {
+					if (pass != grfcontversion) {
+						writespritesize(action, 4, grfcontversion, grf);
+						fputc(0xfd, grf);
+						writedword(action, i + 1, grf);
+						break;
+					}
+
+					info.PrepareReal(sprite.infs[j]);
+					CommonPixel *image = (CommonPixel*) calloc(info.imgsize, sizeof(CommonPixel));
+					if (!image) {
+						fprintf(stderr, "%s:%d: Error: can't allocate sprite memory (%ld bytes)\n", file, i, info.imgsize);
+						exit(2);
+					}
+
+					bool has_mask=sprite.infs[j].depth==DEPTH_8BPP;
+					bool rgba=sprite.infs[j].depth==DEPTH_32BPP;
+					info.getsprite(image);
+					if(j+1<sprite.infs.size()&&sprite.infs[j+1].depth==DEPTH_MASK){
+						j++;
+						info.PrepareReal(sprite.infs[j]);
+						CommonPixel *mask = (CommonPixel*) calloc(info.imgsize, sizeof(CommonPixel));
+						if (!mask) {
+							fprintf(stderr, "%s:%d: Error: can't allocate sprite memory (%ld bytes)\n", file, i, info.imgsize);
+							exit(2);
+						}
+						info.getsprite(mask);
+						for (int i = 0; i < info.imgsize; i++) image[i].m = mask[i].m;
+						free(mask);
+						has_mask=true;
+					}
+
+					int k=0;
+					for (int j=info.imgsize-1; j >= 0; j--)
+						if (has_mask && image[j].m == 0xFF) k++;
+
+					if (k && !_quiet)
+						fprintf(stderr, "%s:%d: Warning: %d of %ld pixels (%ld%%) are pure white\n",
+							file, i, k, info.imgsize, k*100/info.imgsize);
+
+					if(_crop && !DONOTCROP(info.inf.info)){
+						int i=0,j=0;
+						for(i=info.imgsize-1;i>=0;i--)if(!image[i].IsTransparent(has_mask, rgba))break; // Find last non-blue pixel
+						if(i<0)// We've got an all-blue sprite
+							info.sx=info.sy=info.imgsize=1;
+						else{
+							i=info.imgsize-(i+info.sx-i%info.sx/*begining of next line*/);
+							info.sy-=i/info.sx;
+
+							for(i=0;i<info.imgsize;i++){
+								if(!image[i].IsTransparent(has_mask, rgba))
+									break; // Find first non-blue pixel
+							}
+							i-=i%info.sx;// Move to beginning of line
+
+							info.sy-=i/info.sx;
+							info.inf.yrel+=i/info.sx;
+							if(i)memmove(image,image+i,(info.imgsize-i)*sizeof(CommonPixel));
+							for(i=0;i<info.sx;i++){
+								for(j=0;j<info.sy;j++){
+									if(!image[i+j*info.sx].IsTransparent(has_mask, rgba))goto foundfirst;
+								}
+							}
+foundfirst:
+							if(i){
+								for(j=0;j<info.sy;j++)
+									memmove(image+j*(info.sx-i),image+j*info.sx+i,(info.sx-i)*sizeof(CommonPixel));
+								info.inf.xrel+=i;
+								info.sx-=i;
+							}
+
+							for(i=info.sx-1;i>=0;i--){
+								for(j=0;j<info.sy;j++){
+									if(!image[i+j*info.sx].IsTransparent(has_mask, rgba))goto foundlast;
+								}
+							}
+foundlast:
+							i=info.sx-i-1;
+							if(i){
+								for(j=1;j<info.sy;j++)
+									memmove(image+j*(info.sx-i),image+j*info.sx,(info.sx-i)*sizeof(CommonPixel));
+								info.sx-=i;
+							}
+
+						}
+						info.inf.xdim = info.sx;
+						info.inf.ydim = info.sy;
+						info.imgsize = info.sx * info.sy;
+					}
+
+					U8 bytes_per_pixel=(has_mask?1:0)+(rgba?4:0);
+					U8 *compressed_chunked = NULL;
+					U8 *compressed_regular = NULL;
+					long compressed_size_chunked = 1 << 30;
+					long compressed_size_regular = 1 << 30;
+					long uncompressed_size_chunked = 0;
+
+					bool force_chunk = HASTRANSPARENCY(info.inf.info);
+					if (force_chunk || _best_compression) {
+						SpriteInfo inf = info.inf;
+						inf.info |= 8; // Set chunked status
+						compressed_size_chunked = encodetile(&compressed_chunked, &uncompressed_size_chunked, image, info.imgsize*bytes_per_pixel, info.sx, info.sy, inf, compress, i, has_mask, rgba, grfcontversion);
+					}
+					if (!force_chunk || _best_compression) {
+						U8 *imgbuffer = (U8*)malloc(info.imgsize*bytes_per_pixel);
+						if (!imgbuffer) {
+							fprintf(stderr, "%s:%d: Error: can't allocate sprite memory (%ld bytes)\n", file, i, info.imgsize);
+							exit(2);
+						}
+						for (int j = 0; j < info.imgsize; j++) {
+							image[j].Encode(imgbuffer + (j * bytes_per_pixel), has_mask, rgba);
+						}
+						SpriteInfo inf = info.inf;
+						inf.info &= ~8; // Clear chunked status
+						compressed_size_regular = encoderegular(&compressed_regular, imgbuffer, info.imgsize*bytes_per_pixel, inf, compress, i, grfcontversion);
+						free(imgbuffer);
+					}
+
+					/* GRF container version 2 saves 4 extra bytes for the chunked data. */
+					bool use_chunk = compressed_size_chunked + (grfcontversion == 2 ? 4 : 0) < compressed_size_regular;
+					long uncompressed_size = use_chunk ? uncompressed_size_chunked : info.imgsize*bytes_per_pixel;
+					long compressed_size   = use_chunk ? compressed_size_chunked : compressed_size_regular;
+					if (use_chunk) {
+						info.inf.info |= 8; // Set chunked status
+					} else {
+						info.inf.info &= ~8; // Clear chunked status
+					}
+					writesprite(grf, use_chunk ? compressed_chunked : compressed_regular, compressed_size, uncompressed_size, info.inf, i, grfcontversion);
+
+					totaltransp += uncompressed_size;	// how much after transparency removed
+					totaluntransp += info.imgsize;		// how much with transparency
+
+					totalreg += compressed_size;			// how much after transp&redund removed
+					totalunreg += uncompressed_size;	// how much with redund
+
+					totalcomp += compressed_size;
+					totaluncomp += info.imgsize;
+					free(compressed_chunked);
+					free(compressed_regular);
+					free(image);
+				}
+			}
+				break;
+			default:
+				fprintf(stderr, "%s:%d: Error: What type of sprite is that?", file, i);
+				exit(2);
+			}
+		}
+
+		writespritesize("writing end-of-file", 0, grfcontversion, grf);
+		if (grfcontversion == 1) {
+			writedword("writing checksum", 0, grf);
+		}
+	}
 
 	fclose(grf);
 
@@ -599,26 +689,45 @@ static int decode(const char *file, const char *dir, const U8 *palette, int box,
 	fsize = ftell(grf);
 	fseek(grf, 0, SEEK_SET);
 
+	static const char *action = "reading header";
+	U8 buffer[sizeof(header)];
+	cfread(action, buffer, 1, sizeof(buffer), grf);
+	int grfcontversion = memcmp(buffer, header, sizeof(header)) == 0 ? 2 : 1;
+
+	U32 dataoffset = 0;
+	if (grfcontversion == 1) {
+		fseek(grf, 0, SEEK_SET);
+	} else {
+		dataoffset = sizeof(header) + 4 + readdword(action, grf); // GRF data offset
+		fgetc(grf); // Compression
+	}
+	printf("Found grf container version %d\n", grfcontversion);
+
 	// We do the 'file' and 'writer' seperate to make
 	//   this a little bit less messy
-	multifile *imgname;
-	pcxwrite *pcx;
+	multifile *imgname, *imgname32;
+	pcxwrite *pcx, *pcx32;
 
-	if (height == -1)
-		imgname = new singlefile(spritefilename(file, dir, getoutputext(), -2, "wb", 1),"wb", dir);
-	else
-		imgname = new spritefiles(file, dir);
+	if (height == -1) {
+		imgname   = new singlefile(spritefilename(file, dir, getoutputext(false), -2, "wb", 1),"wb", dir);
+		imgname32 = new singlefile(spritefilename(file, dir, getoutputext(true),  -2, "wb", 1),"wb", dir);
+	} else {
+		imgname   = new spritefiles(file, dir, false);
+		imgname32 = new spritefiles(file, dir, true);
+	}
 
 	// Select the appropriate writer
 	switch (_outputformat) {
 #ifdef WITH_PNG
 		case SSF_PNG:
-			pcx = new pngwrite(imgname);
+			pcx   = new pngwrite(imgname, false);
+			pcx32 = new pngwrite(imgname32, true);
 			break;
 #endif
 		case SSF_PCX:
 		default:
-			pcx = new pcxwrite(imgname);
+			pcx   = new pcxwrite(imgname);
+			pcx32 = NULL;
 			break;
 	}
 
@@ -630,13 +739,15 @@ static int decode(const char *file, const char *dir, const U8 *palette, int box,
 	pcx->setpalette(palette);
 
 	pcx->setcolours(255, 0, 0);
+	if (pcx32 != NULL) pcx32->setcolours(255, 0, 0);
 
-	infowriter writer(info, (width + box - 1) / box, useplaintext);
+	infowriter writer(info, (width + box - 1) / box, useplaintext, pcx->getdirectory());
 
 	if (colourmap)
 		pcx->installwritemap(colourmap);
 
-	pcx->startimage(width, height, box, box, &writer);
+	pcx->startimage(true, width, height, box, box);
+	if (pcx32 != NULL) pcx32->startimage(false, width, height, box, box);
 
 	count = 0;
 
@@ -650,14 +761,22 @@ static int decode(const char *file, const char *dir, const U8 *palette, int box,
 			printf("Sprite %d at %lX, %3d%% done\r", count, ftell(grf), lastpct);
 		}
 
+		pcx->newsprite();
+		if (pcx32 != NULL) pcx32->newsprite();
+
+		result = decodesprite(grf, pcx, pcx32, &writer, count, &dataoffset, grfcontversion);
+		writer.flush();
 		count++;
-		result = decodesprite(grf, pcx, &writer);
 	} while (result);
 	count--;
 
 	pcx->endimage();
+	if (pcx32 != NULL) pcx32->endimage();
 	delete(pcx);	// closes output file
+	delete(pcx32);
+	if (pcx32 == NULL) delete imgname32;
 
+	writer.flush();
 	writer.done(count);
 
 	fclose(info);
@@ -798,6 +917,7 @@ int main(int argc, char **argv)
 	U8 *palette = NULL;
 	int *colourmap = NULL;
 	int useplaintext = 1;
+	int grfcontversion = 1;
 
 	_interactive = (isatty(fileno(stdout)) != 0);
 
@@ -809,7 +929,7 @@ int main(int argc, char **argv)
 
 	// parse option arguments
 	while (1) {
-		char opt = getopt(argc, argv, "dev?w:h:b:up:m:M:o:tfxqcsX");
+		char opt = getopt(argc, argv, "dev?w:h:b:up:m:M:o:tfxqcsXg:n");
 
 		if (opt == (char) EOF)
 			break;
@@ -833,6 +953,9 @@ int main(int argc, char **argv)
 		case 'b':
 			box = atoi(optarg);
 			break;
+		case 'n':
+			_best_compression = true;
+			break;
 		case 'p':
 			unsigned int palnum;
 			palnum = atoi(optarg);
@@ -848,6 +971,10 @@ int main(int argc, char **argv)
 		case 'c':
 			_crop++;
 			break;
+		case 'g':
+			grfcontversion = atoi(optarg);
+			if (grfcontversion < 1 || grfcontversion > 2) usage();
+			break;
 		case 'm':
 		case 'M':
 			_mapAll= opt=='M';
@@ -914,7 +1041,7 @@ int main(int argc, char **argv)
 		usage();
 
 	if (action == 1) {
-		return encode(grffile, directory, compress, colourmap);
+		return encode(grffile, directory, compress, colourmap, grfcontversion);
 	} else if (action == 2) {
 		if (!palette)
 			palette = findpal(grffile);
