@@ -22,8 +22,8 @@ class inforeader {
 	long imgsize;
 	int sx, sy;
 	SpriteInfo inf;
-	void PrepareReal(const Real&);
-	int getsprite(U8 *sprite);
+	void PrepareReal(const SpriteInfo&info);
+	int getsprite(CommonPixel *sprite);
 
 	pcxread *imgfile;
 
@@ -36,38 +36,45 @@ private:
 	pcxread* MakeReader()const;
 };
 
+struct Box {
+	enum boxtype {issprite, isspriteextension, isdata};
+	Box(boxtype type) : type(type) {}
+	virtual ~Box() {}
+	boxtype type;
+};
+
+struct BoxData : Box {
+	BoxData(U16 size, U8 *data) : Box(isdata), size(size), data(data) {}
+	U16 size;
+	U8 *data;
+};
+
+struct BoxSprite : Box {
+	BoxSprite(bool first, const char *filename, int x, int y, SpriteInfo info) : Box(first ? issprite : isspriteextension), filename(strdup(filename)), x(x), y(y), info(info) {}
+	~BoxSprite() { free(filename); }
+	char *filename;
+	int x;
+	int y;
+	SpriteInfo info;
+};
+
 class infowriter :  public spriteinfowriter {
 	public:
-	infowriter(FILE *info, int maxboxes, int useplaintext);
+	infowriter(FILE *info, int maxboxes, int useplaintext, const char *directory);
 	virtual ~infowriter();
 
-	virtual void newband(pcxfile *pcx);
-	virtual void addsprite(int x, SpriteInfo info);
+	void flush();
+	virtual void addsprite(bool first, const char *filename, int x, int y, SpriteInfo info);
 	virtual void adddata(U16 size, U8 *data);
 
 	void done(int count);
 
 	private:
 	FILE *info;
-
+	const char *directory;
 	void resize(int newmaxboxes);
+	Box **boxes;
 
-	enum boxtype {isinvalid, issprite, isdata};
-
-	struct box {
-		boxtype type;
-		union foo {
-			struct boxsprite {
-				int x;
-				SpriteInfo info;
-			} sprite;
-
-			struct boxdata {
-				U16 size;
-				U8 *data;
-			} data;
-		} h;
-	} *boxes;
 	int spriteno, maxboxes, boxnum;
 	int useplaintext;
 };
